/***************************************************************************
 *   copyright           : (C) 2002 by Hendrik Sattler                     *
 *   mail                : post@hendrik-sattler.de                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "charsets.h"
#include "helpers.h"
#include "common.h"

#include <stdio.h>
#include <string.h>

size_t ucs2len (ucs2char_t* input) {
  size_t retval = 0;
  if (input != 0) {
    while (*input != 0) {
      ++input;
      ++retval;
    }
  }
  return retval;
}

#include "../config.h"
#ifdef WORDS_BIGENDIAN
#  define UCS2_CHARSET "UCS-2BE"
#else
#  define UCS2_CHARSET "UCS-2LE"
#endif

ucs2char_t* convert_to_ucs2 (ucs4char_t* input) {
  return (ucs2char_t*)convert_from_internal(UCS2_CHARSET,
					    input,
					    REPMODE_ESCAPE_CHARS);
}

ucs4char_t* convert_from_ucs2 (ucs2char_t* input) {
  return convert_to_internal(UCS2_CHARSET,
			     (unsigned char*)input,
			     ucs2len(input)*sizeof(ucs2char_t));
}


char* convert_to_ucs2_hexstring (ucs4char_t* input) {
  char* retval;
  ucs2char_t* temp;
  int i;

  temp = convert_to_ucs2(input);
  retval = mem_alloc((ucs2len(temp)*2*sizeof(ucs2char_t))+1,1);
  for (i=0;i<ucs2len(temp);i++) {
    sprintf(retval+(4*i),"%04X",temp[i]);
  }
  mem_realloc(temp,0);
  return retval;
}

ucs4char_t* convert_from_ucs2_hexstring (char* input) {
  ucs4char_t* retval;
  ucs2char_t* temp;
  int i;
    
  temp = mem_alloc(((strlen(input)/4)+(strlen(input)%4)+1)*sizeof(ucs2char_t),1);
  for (i=0;(i*4)<strlen(input);i++) {
    temp[i]=hexstr2int(input+(i*4),4);
  }

  retval = convert_from_ucs2(temp);
  mem_realloc(temp,0);
  return retval;
}
